({
  currentGlobalId,
  roleTagMap,
  processIFrames = true,
  includeAriaHidden = false,
  nodeIdsToIgnore = [],
}) => {
  function generate_unqie_dom_id() {
    currentGlobalId++;
    window["tf_generated_dom_id"] = currentGlobalId;
    return currentGlobalId;
  }

  function buildAccessibilityTree() {
    const rootNode = document.documentElement;
    const tree = createNode(rootNode);
    return tree;
  }

  function extractAttributes(node) {
    const attributes = { html_tag: node.nodeName.toLowerCase() };
    const skippedAttributes = ["style", "srcdoc"];
    for (let i = 0; i < node.attributes.length; i++) {
      const attribute = node.attributes[i];
      if (!attribute.specified || !skippedAttributes.includes(attribute.name)) {
        attributes[attribute.name] = attribute.value || true;
      }
    }
    return attributes;
  }

  function getRole(node) {
    const role = node.getAttribute("role") || node.getAttribute("aria-role");
    if (role) {
      return role;
    }
    const tag = node.nodeName.toLowerCase();
    if (tag == "input") {
      const type = node.getAttribute("type");
      if (type) {
        return roleTagMap.input[type] || type;
      }
    }
    return roleTagMap.role[tag] || "generic";
  }

  function getName(element) {
    // Special case: textContent for button elements
    if (element.tagName.toLowerCase() === "button") {
      const textContent = element.textContent.trim();
      if (textContent) {
        return textContent;
      }
    }

    // List of attributes in the order they should be checked
    const attributes = [
      "aria-label",
      "placeholder",
      "alt",
      "title",
      "value",
      "name",
    ];

    // Iterate over the attributes and return the first non-empty one
    for (const attribute of attributes) {
      if (element.hasAttribute(attribute)) {
        const value = element.getAttribute(attribute).trim();
        if (value) {
          return value;
        }
      }
    }

    // Fallback to empty string if no non-empty attribute is found
    return "";
  }

  function isElementHidden(element) {
    const style = window.getComputedStyle(element);

    return (
      style.display === "none" ||
      style.visibility === "hidden" ||
      element.hasAttribute("hidden")
    );
  }

  function createNode(node) {
    const ariaHidden =
      node.getAttribute("aria-hidden") === "true" ||
      node.getAttribute("aria-hidden") === true;
    const tag = node.nodeName.toLowerCase();
    const skippedTags = [
      "script",
      "style",
      "path",
      "g",
      "hr",
      "br",
      "noscript",
      "head",
    ];
    if (skippedTags.includes(tag) || nodeIdsToIgnore.includes(node.id)) {
      return null;
    }
    let role = getRole(node);
    let name = getName(node) || "";

    let children = [];
    let currentChildNodes = node.childNodes;
    if (node.shadowRoot) {
      const shadowRootChildren = node.shadowRoot.children;
      const childrenNodeList = Array.prototype.slice.call(shadowRootChildren);

      if (childrenNodeList.length > 0) {
        currentChildNodes = Array.from(childrenNodeList);
      } else if (node.shadowRoot.textContent.trim() !== "") {
        name = node.shadowRoot.textContent.trim();
      }
    } else if (tag == "slot") {
      currentChildNodes = node.assignedNodes();
    } else if (tag == "iframe") {
      const iframeDocument = node.contentDocument;
      if (iframeDocument && processIFrames) {
        if (iframeDocument.body) {
          currentChildNodes = iframeDocument.body.childNodes;
        } else {
          currentChildNodes = iframeDocument.childNodes;
        }
      } else if (node.src) {
        role = "iframe"; // iframe without contentDocument
      }
    }
    const childNodes = Array.from(currentChildNodes).filter((childNode) => {
      return (
        childNode.nodeType === Node.ELEMENT_NODE ||
        (childNode.nodeType === Node.TEXT_NODE &&
          childNode.textContent.trim() !== "")
      );
    });
    for (let i = 0; i < childNodes.length; i++) {
      let childNode = childNodes[i];
      if (childNode.nodeType === Node.TEXT_NODE) {
        const text = childNode.textContent.trim();
        if (text) {
          if (childNodes.length > 1) {
            const span = document.createElement("span");
            span.textContent = text;
            if (node.contains(childNode)) {
              node.insertBefore(span, childNode);
              node.removeChild(childNode);
            }
            childNode = span;
          } else if (childNodes.length == 1) {
            if (!role || role == "generic") {
              role = "text";
            }
            name = text;
          }
        }
      }
      if (childNode.nodeType === Node.ELEMENT_NODE) {
        const child = createNode(childNode);
        if (child) {
          children.push(child);
        }
      }
    }
    if (children.length === 0) {
      children = undefined;
    }

    if (!role && !name && !children) {
      return null;
    } else {
      let nodeId = generate_unqie_dom_id();
      if (!node.hasAttribute("tf623_id")) {
        node.setAttribute("tf623_id", nodeId);
      }
      if (!isElementHidden(node) && (includeAriaHidden || !ariaHidden)) {
        let node_dict = {
          role: role || "generic",
          name,
          attributes: extractAttributes(node),
        };
        if (children) {
          node_dict["children"] = children;
        }
        return node_dict;
      } else {
        return null;
      }
    }
  }

  return {
    tree: buildAccessibilityTree(),
    lastUsedId: currentGlobalId,
  };
};
